@@ -5,7 +5,7 @@ import logging
 import random
 import time
 from collections import ChainMap, abc, defaultdict, namedtuple
-from contextvars import ContextVar
+from contextvars import ContextVar, copy_context
 from functools import partial
 from itertools import chain
 from typing import Any, Collection, List, Mapping, Optional, Tuple, Union
@@ -31,15 +31,26 @@ from .config import (
     is_abort,
     is_debug,
     is_endure_operations,
+    is_layered_solution,
     is_marshal_tasks,
     is_parallel_tasks,
     is_reschedule_operations,
     is_skip_evictions,
 )
-from .modifiers import dep_singularized, dep_stripped, is_sfx
-from .network import (
-    _EvictInstruction,
+from .modifier import (
+    acc_contains,
+    acc_delitem,
+    acc_getitem,
+    acc_setitem,
+    dep_singularized,
+    dep_stripped,
+    get_accessor,
+    get_jsonp,
+    is_sfx,
+)
+from .planning import (
     unsatisfied_operations,
+    yield_chaindocs,
     yield_node_names,
     yield_ops,
 )
@@ -57,16 +68,29 @@ def _isDebugLogging():
 class Solution(ChainMap, Plottable):
     """
     The :term:`solution` chain-map and execution state (e.g. :term:`overwrite` or :term:`canceled operation`)
+
+    It inherits :class:`collections.ChainMap`, to keep a separate dictionary
+    for each operation executed, +1 for the user inputs.
     """
 
-    def __init__(self, plan, input_values: dict):
-        #: An ordered mapping of plan-operations to their results
+    def __init__(self, plan, input_values: dict, layered_solution=None):
+        layered_solution = first_solid(is_layered_solution(), layered_solution)
+        if layered_solution is None:
+            layered_solution = not any(get_jsonp(d) for d in plan.net.data)
+
+        #: :term:`layer` map of operations to their results
         #: (initially empty dicts).
         #: The result dictionaries pre-populate this (self) chainmap,
-        #: with the 1st map (wins all reads) the last operation,
+        #: with the 1st map (wins all reads) the last operation executed,
         #: the last one the `input_values` dict.
-        self._layers = {op: {} for op in yield_ops(reversed(plan.steps))}
-        super().__init__(*self._layers.values(), input_values)
+        #:
+        #: If network contains :term:`jsonp` dependencies, by default
+        #: layers are disabled, and this is None.
+        self._layers: Optional[Mapping] = {
+            op: {} for op in yield_ops(reversed(plan.steps))
+        } if layered_solution else None
+        maps = self._layers.values() if self._layers else ()
+        super().__init__(*maps, input_values)
 
         #: the plan that produced this solution
         self.plan = plan
@@ -79,9 +103,6 @@ class Solution(ChainMap, Plottable):
         self.executed = {}
         #: A sorted set of :term:`canceled operation`\\s due to upstream failures.
         self.canceled = iset()  # not iterated, order not important, but ...
-        #: a flag controlled by `plan` (by invoking :meth:`finalized` is invoked)
-        #: which becomes `True` when this instance has finished accepting results.
-        self.finalized = False
         self.elapsed_ms = {}
         #: A unique identifier to distinguish separate flows in execution logs.
         self.solid = "%X" % random.randint(0, 2 ** 16)
@@ -105,7 +126,7 @@ class Solution(ChainMap, Plottable):
     def __copy__(self):
         clone = type(self)(self.plan, {})
         props = (
-            "maps executed canceled finalized elapsed_ms solid _layers"
+            "maps executed canceled elapsed_ms solid _layers"
             " is_endurance is_reschedule is_parallel is_marshal dag"
         ).split()
         for p in props:
@@ -123,22 +144,33 @@ class Solution(ChainMap, Plottable):
     def debugstr(self):
         return f"{type(self).__name__}({dict(self)}, {self.plan}, {self.executed}, {self._layers})"
 
-    def _reschedule(self, dag, nbunch_to_break, op):
-        """Update dag/canceled/executed ops and return newly-canceled ops. """
-        dag.remove_edges_from(tuple(dag.out_edges(nbunch_to_break)))
-        canceled = unsatisfied_operations(dag, self)
+    def _reschedule(self, dag, reason, op):
+        """
+        Re-prune dag, and then update and return any newly-canceled ops.
+
+        :param dag:
+            The dag to discover :term:`unsatisfied operation`\\s from.
+        :param reason:
+            for logging
+        :param op:
+            for logging
+        """
+        canceled = unsatisfied_operations(
+            dag,
+            [
+                i
+                for i in self
+                # don't send canceled SFXs as Inputs.
+                if not is_sfx(i) or self.get(i, True)
+            ],
+        )
         # Minus executed, bc partial-out op might not have any provides left.
         newly_canceled = iset(canceled) - self.canceled - self.executed
         self.canceled.update(newly_canceled)
 
-        if newly_canceled and log.isEnabledFor(logging.INFO):
-            reason = (
-                "failure of"
-                if isinstance(nbunch_to_break, Operation) > 1
-                else "rescheduled"
-            )
+        if log.isEnabledFor(logging.INFO):
             log.info(
-                "... (%s) CANCELING +%s ops%s due to %s of op(%s).",
+                "... (%s) +%s  newly CANCELED ops%s due to %s op(%s).",
                 self.solid,
                 len(newly_canceled),
                 [n.name for n in newly_canceled],
@@ -146,6 +178,54 @@ class Solution(ChainMap, Plottable):
                 op.name,
             )
 
+    def _update_op_outs(self, op, outputs: Mapping):
+        """
+        Mass update values on the :term:`solution layer` for the given `op`.
+
+        Observes any :term:`accessor`\\s in the k
+        A separate method to allow subclasses with custom accessor logic.
+        """
+        op_layer = self._layers[op] if self._layers else self.maps[-1]
+        accessors = [get_accessor(o) for o in outputs]
+        if any(accessors):
+            ## Group out-values by their `update` accessor (or None).
+            update_groups = defaultdict(list)
+            for ac, kv in zip(accessors, outputs.items()):
+                update_groups[ac and ac.update].append(kv)
+
+            ## Values without :attr:`Accessor.update` are to be set one-by-one,
+            #  further below.
+            outputs = update_groups.pop(None, None)
+
+            for upd, pairs in update_groups.items():
+                upd(op_layer, pairs)
+
+        if outputs:
+            op_layer.update(outputs)
+
+    def __contains__(self, key):
+        acc = acc_contains(key)
+        return any(acc(m, key) for m in self.maps)
+
+    def __getitem__(self, key):
+        acc = acc_getitem(key)
+        for mapping in self.maps:
+            try:
+                return acc(mapping, key)
+            except KeyError:
+                pass
+        return self.__missing__(key)
+
+    def __delitem__(self, key):
+        acc = acc_contains(key)
+        matches = [m for m in self.maps if acc(m, key)]
+        if not matches:
+            raise KeyError(key)
+
+        acc = acc_delitem(key)
+        for m in matches:
+            acc(m, key)
+
     def operation_executed(self, op, outputs):
         """
         Invoked once per operation, with its results.
@@ -168,8 +248,7 @@ class Solution(ChainMap, Plottable):
                 return ()
             return dep_singularized(dep)
 
-        assert not self.finalized, f"Cannot reuse solution: {self}"
-        self._layers[op].update(outputs)
+        self._update_op_outs(op, outputs)
         self.executed[op] = None
 
         if first_solid(self.is_reschedule, op.rescheduled):
@@ -183,18 +262,20 @@ class Solution(ChainMap, Plottable):
                 for k, v in outputs.items()
                 for sf in collect_canceled_sideffects(k, v)
             ]
-            to_break = (missing_outs - sfx) | canceled_sideffects
+            outs_to_break = (missing_outs - sfx) | canceled_sideffects
             log.info(
                 "... (%s) missing partial outputs %s from rescheduled %s.",
                 self.solid,
-                list(to_break),
+                list(outs_to_break),
                 op,
             )
 
-            if to_break:
-                self._reschedule(self.dag, to_break, op)
+            if outs_to_break:
+                dag = self.dag
+                dag.remove_edges_from((op, out) for out in outs_to_break)
+                self._reschedule(dag, "rescheduled", op)
                 # list used by `check_if_incomplete()`
-                self.executed[op] = to_break
+                self.executed[op] = outs_to_break
 
     def operation_failed(self, op, ex):
         """
@@ -203,19 +284,10 @@ class Solution(ChainMap, Plottable):
         It will update :attr:`executed` with the operation status and
         the :attr:`canceled` with the unsatisfied ops downstream of `op`.
         """
-        assert not self.finalized, f"Cannot reuse solution: {self}"
+        dag = self.dag
         self.executed[op] = ex
-        self._reschedule(self.dag, op, op)
-
-    def finalize(self):
-        """invoked only once, after all ops have been executed"""
-        # Invert solution so that last value wins
-        if not self.finalized:
-            self.finalized = True
-
-    def __delitem__(self, key):
-        for d in self.maps:
-            d.pop(key, None)
+        dag.remove_edges_from(tuple(dag.out_edges(op)))
+        self._reschedule(dag, "failure of", op)
 
     def is_failed(self, op):
         return isinstance(self.executed.get(op), Exception)
@@ -226,10 +298,8 @@ class Solution(ChainMap, Plottable):
         The data in the solution that exist more than once.
 
         A "virtual" property to a dictionary with keys the names of values that
-        exist more than once, and values, all those values in a list, ordered:
-
-        - before :meth:`finalized()`, as computed;
-        - after :meth:`finalized()`, in reverse.
+        exist more than once, and values, all those values in a list, ordered
+        in reverse compute order (1st is the last one computed).
         """
         maps = self.maps
         dd = defaultdict(list)
@@ -269,6 +339,10 @@ class Solution(ChainMap, Plottable):
         if ex:
             raise ex
 
+    @property
+    def graph(self):
+        return self.dag
+
     def prepare_plot_args(self, plot_args: PlotArgs) -> PlotArgs:
         """delegate to plan, with solution"""
         name = f"solution-x{len(self.plan.net.graph.nodes)}-nodes"
@@ -307,8 +381,6 @@ class _OpTask:
             log.debug("+++ (%s) Executing %s...", self.solid, self)
             token = task_context.set(self)
             try:
-                ## Not really needed ...
-                # self.result = copy_context().run(self.op.compute, self.sol)
                 self.result = self.op.compute(self.sol)
             finally:
                 task_context.reset(token)
@@ -327,6 +399,9 @@ class _OpTask:
 
 #: Populated with the :class:`_OpTask` for the currently executing operation.
 #: It does not work for :term:`parallel execution`.
+#:
+#: .. seealso::
+#:     The elaborate example in :ref:`hierarchical-data` section
 task_context: ContextVar[_OpTask] = ContextVar("task_context")
 
 
@@ -343,7 +418,7 @@ def _do_task(task):
         import dill
 
         task = dill.loads(task)
-        result = task()
+        result = copy_context().run(task)
         result = dill.dumps(result)
     else:
         result = task()
@@ -357,7 +432,7 @@ class ExecutionPlan(
     """
     A pre-compiled list of operation steps that can :term:`execute` for the given inputs/outputs.
 
-    It is the result of the network's :term:`compilation` phase.
+    It is the result of the network's :term:`planning` phase.
 
     Note the execution plan's attributes are on purpose immutable tuples.
 
@@ -380,10 +455,14 @@ class ExecutionPlan(
         any given intermediate inputs.
     .. attribute:: asked_outs
 
-        When true, :term:`evictions` may kick in (unless disabled by :term:`configurations`),
+        When true, :term:`eviction`\\s may kick in (unless disabled by :term:`configurations`),
         otherwise, *evictions* (along with prefect-evictions check) are skipped.
     """
 
+    @property
+    def graph(self):
+        return self.dag
+
     def prepare_plot_args(self, plot_args: PlotArgs) -> PlotArgs:
         plot_args = plot_args.clone_or_merge_graph(self.net.graph)
         graph = plot_args.graph
@@ -532,16 +611,20 @@ class ExecutionPlan(
         except Exception as ex:
             is_endured = first_solid(solution.is_endurance, op.endured)
             elapsed = elapsed_ms(op)
-            loglevel = logging.WARNING if is_endured else logging.DEBUG
+            loglevel = logging.WARNING if is_endured else logging.ERROR
             log.log(
                 loglevel,
-                "... (%s) %s op(%s) FAILED in %0.3fms, due to: %s(%s)",
+                "... (%s) %s op(%s) FAILED in %0.3fms, due to: %s(%s)"
+                "\n  x%i ops executed so far: %s",
                 solution.solid,
                 "*Enduring* " if is_endured else "",
                 op.name,
                 elapsed,
                 type(ex).__name__,
                 ex,
+                len(solution.executed),
+                solution.executed,
+                exc_info=is_debug(),
             )
 
             if is_endured:
@@ -589,7 +672,7 @@ class ExecutionPlan(
                 ):
                     if node not in solution.canceled:
                         upnext.append(node)
-                elif isinstance(node, _EvictInstruction):
+                elif isinstance(node, str):
                     # Only evict if all successors for the data node
                     # have been executed.
                     if (
@@ -622,7 +705,7 @@ class ExecutionPlan(
                 #  TODO: evictions for parallel are wasting loops.
                 #
                 for node in self.steps:
-                    if isinstance(node, _EvictInstruction) and node in solution:
+                    if isinstance(node, str) and node in solution:
                         del solution[node]
                 break
 
@@ -657,7 +740,7 @@ class ExecutionPlan(
                 task = _OpTask(step, solution, solution.solid)
                 self._handle_task(task, step, solution)
 
-            elif isinstance(step, _EvictInstruction):
+            elif isinstance(step, str):
                 # Cache value may be missing if it is optional.
                 if step in solution:
                     log.debug(
@@ -672,7 +755,13 @@ class ExecutionPlan(
                 raise AssertionError(f"Unrecognized instruction.{step}")
 
     def execute(
-        self, named_inputs, outputs=None, *, name="", solution_class=None
+        self,
+        named_inputs,
+        outputs=None,
+        *,
+        name="",
+        solution_class=None,
+        layered_solution=None,
     ) -> Solution:
         """
         :param named_inputs:
@@ -687,6 +776,16 @@ class ExecutionPlan(
             name of the pipeline used for logging
         :param solution_class:
             a custom solution factory to use
+        :param layered_solution:
+            whether to store operation results into separate :term:`solution layer`
+
+            Unless overridden by a True/False in :func:`.set_layered_solution`
+            of :term:`configurations`, it accepts the following values:
+
+            - When True(False), always keep(don't keep) results in a separate layer for each operation,
+              regardless of any *jsonp* dependencies.
+            - If ``None``, layers are used only if there are NO :term:`jsonp` dependencies
+              in the network.
 
         :return:
             The :term:`solution` which contains the results of each operation executed
@@ -707,6 +806,7 @@ class ExecutionPlan(
         """
         try:
             self.validate(named_inputs, outputs)
+            dag = self.dag  # locals opt
 
             ## Choose a method of execution
             #
@@ -730,9 +830,10 @@ class ExecutionPlan(
 
             solution = solution_class(
                 self,
-                {k: v for k, v in named_inputs.items() if k in self.dag.nodes}
+                {k: v for k, v in named_inputs.items() if k in dag.nodes}
                 if evict
                 else named_inputs,
+                layered_solution=layered_solution,
             )
 
             log.debug(
@@ -745,18 +846,19 @@ class ExecutionPlan(
                 self,
             )
 
+            ok = False
             try:
                 executor(solution)
+                ok = True
             finally:
-                solution.finalize()
-
                 ## Log cumulative operations elapsed time.
                 #
                 if _isDebugLogging():
                     elapsed = sum(solution.elapsed_ms.values())
                     log.debug(
-                        "=== (%s) Completed pipeline(%s) in %0.3fms.",
+                        "=== (%s) %s pipeline(%s) in %0.3fms.",
                         solution.solid,
+                        "Completed" if ok else "FAILED",
                         name,
                         elapsed,
                     )
@@ -764,11 +866,16 @@ class ExecutionPlan(
             # Validate eviction was perfect
             #
             if evict:
-                expected_provides = set(dep_stripped(n) for n in self.provides)
+                expected_provides = set()
+                expected_provides.update(
+                    yield_chaindocs(dag, self.provides, expected_provides,)
+                )
+                expected_provides = set(dep_stripped(n) for n in expected_provides)
                 # It is a proper subset when not all outputs calculated.
                 assert set(solution).issubset(expected_provides), (
                     f"Evictions left more data{list(iset(solution) - set(self.provides))} than {self}!"
-                    ' \n Did you bypass "impossible-outputs" validation?'
+                    '\n  (hint: did you bypass "impossible-outputs" validation?)'
+                    "\n  (tip: enable DEBUG-logging and/or set GRAPHTIK_DEBUG envvar to investigate)"
                 )
 
             return solution
