@@ -1,10 +1,10 @@
 # Copyright 2016, Yahoo Inc.
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 """
-:term:`compose` operations into :term:`pipeline` and the backing :term:`network`.
+:term:`compose` :term:`pipeline`\\s by :term:`combining <combine pipelines>` operations into :term:`network`.
 
 .. note::
-    This module (along with :mod:`.op` & :mod:`.modifiers`) is what client code needs
+    This module (along with :mod:`.op` & :mod:`.modifier`) is what client code needs
     to define pipelines *on import time* without incurring a heavy price
     (<5ms on a 2019 fast PC)
 """
@@ -14,8 +14,19 @@ import re
 from collections import abc as cabc
 from typing import Callable, List, Mapping, Union
 
-from .base import UNSET, Items, Operation, PlotArgs, Plottable, RenArgs, aslist, jetsam
-from .modifiers import dep_renamed
+from boltons.setutils import IndexedSet as iset
+
+from .base import (
+    UNSET,
+    Items,
+    Operation,
+    PlotArgs,
+    Plottable,
+    RenArgs,
+    aslist,
+    jetsam,
+)
+from .modifier import dep_renamed
 
 log = logging.getLogger(__name__)
 
@@ -54,13 +65,76 @@ def _id_tristate_bool(b):
     return 3 if b is None else (hash(bool(b)) + 1)
 
 
-class Pipeline(Operation, Plottable):
+def build_network(
+    operations,
+    rescheduled=None,
+    endured=None,
+    parallel=None,
+    marshalled=None,
+    node_props=None,
+    renamer=None,
+):
+    """
+    The :term:`network` factory that does :term:`operation merging` before constructing it.
+
+    :param nest:
+        see same-named param in :func:`.compose`
+    """
+    kw = {
+        k: v for k, v in locals().items() if v is not None and k not in ("operations")
+    }
+
+    def proc_op(op, parent=None):
+        """clone FuncOperation with certain props changed"""
+        ## Convey any node-props specified in the pipeline here
+        #  to all sub-operations.
+        #
+        if kw:
+            op_kw = kw.copy()
+
+            if node_props:
+                op_kw["node_props"] = {**op.node_props, **node_props}
+
+            if callable(renamer):
+
+                def parent_wrapper(ren_args: RenArgs) -> str:
+                    # Provide RenArgs.parent.
+                    return renamer(ren_args._replace(parent=parent))
+
+                op_kw["renamer"] = parent_wrapper
+            op = op.withset(**op_kw)
+
+        ## Last minute checks
+        #  (or Plot will fail later).
+        #
+        if hasattr(op, "fn"):
+            ## Could not check earlier due to builder pattern.
+            op.validate_fn_name()
+        return op
+
+    merge_set = iset()  # Preseve given node order.
+    for op in operations:
+        if isinstance(op, Pipeline):
+            merge_set.update(proc_op(s, op) for s in op.ops)
+        else:
+            merge_set.add(proc_op(op))
+    merge_set = iset(i for i in merge_set if not isinstance(i, NULL_OP))
+
+    assert all(bool(n) for n in merge_set)
+
+    from .planning import Network  # Imported here not to affect locals() at the top.
+
+    return Network(*merge_set)
+
+
+class Pipeline(Operation):
     """
     An operation that can :term:`compute` a network-graph of operations.
 
     .. Tip::
-        Use :func:`compose()` factory to prepare the `net` and build
-        instances of this class.
+        - Use :func:`compose()` factory to prepare the `net` and build
+          instances of this class.
+        - See :term:`diacritic`\\s to understand printouts of this class.
     """
 
     #: The name for the new pipeline, used when nesting them.
@@ -95,8 +169,7 @@ class Pipeline(Operation, Plottable):
 
                 *Operations may only be added once, ...*
         """
-        from .network import build_network
-        from .op import reparse_operation_data
+        from .fnop import reparse_operation_data
 
         ## Set data asap, for debugging, although `net.withset()` will reset them.
         self.name = name
@@ -134,13 +207,6 @@ class Pipeline(Operation, Plottable):
             items.append(f"x{len(ops)} ops: {steps}")
         return f"{clsname}({', '.join(items)})"
 
-    @property
-    def ops(self) -> List[Operation]:
-        """A new list with all :term:`operation`\\s contained in the :term:`network`."""
-        from .network import yield_ops
-
-        return list(yield_ops(self.net.graph))
-
     def withset(
         self,
         outputs: Items = UNSET,
@@ -182,7 +248,7 @@ class Pipeline(Operation, Plottable):
             mark all contained `operations` to be :term:`marshalled <marshalling>`
             (usefull when run in `parallel` with a :term:`process pool`).
         :param renamer:
-            see respective parameter in :meth:`.FunctionalOperation.withset()`.
+            see respective parameter in :meth:`.FnOp.withset()`.
 
         :return:
             A narrowed pipeline clone, which **MIGHT be empty!***
@@ -231,6 +297,10 @@ class Pipeline(Operation, Plottable):
             renamer=renamer,
         )
 
+    @property
+    def graph(self):
+        return self.net.graph
+
     def prepare_plot_args(self, plot_args: PlotArgs) -> PlotArgs:
         """Delegate to network. """
         from .plot import graphviz_html_string
@@ -294,12 +364,13 @@ class Pipeline(Operation, Plottable):
         outputs: Items = UNSET,
         predicate: "NodePredicate" = None,
         solution_class: "Type[Solution]" = None,
+        layered_solution=None,
     ) -> "Solution":
         """
         Compile a plan & :term:`execute` the graph, sequentially or parallel.
 
         .. Attention::
-            If intermediate :term:`compilation` is successful, the "global
+            If intermediate :term:`planning` is successful, the "global
             :term:`abort run` flag is reset before the :term:`execution` starts.
 
         :param named_inputs:
@@ -312,6 +383,17 @@ class Pipeline(Operation, Plottable):
             filter-out nodes before compiling
         :param solution_class:
             a custom solution factory to use
+        :param layered_solution:
+            whether to store operation results into separate :term:`solution layer`\\s
+
+            Unless overridden by a True/False in :func:`.set_layered_solution`
+            of :term:`configurations`, it accepts the following values:
+
+            - When True(False), always keep(don't keep) results in a separate layer for each operation,
+              regardless of any *jsonp* dependencies.
+            - If ``None``, layers are used only if there are NO :term:`jsonp` dependencies
+              in the network.
+
 
         :return:
             The :term:`solution` which contains the results of each operation executed
@@ -347,15 +429,22 @@ class Pipeline(Operation, Plottable):
             if not predicate:
                 predicate = self.predicate
 
-            # Build the execution plan.
-            log.debug("=== Compiling pipeline(%s)...", self.name)
+            log.debug("=== Compiling %s...", self)
+            # # Discover nested inputs given.
+            # plan = net.compile(
+            #     self._nested_inputs_given(named_inputs), outputs, predicate
+            # )
             plan = net.compile(named_inputs.keys(), outputs, predicate)
 
             # Restore `abort` flag for next run.
             reset_abort()
 
             solution = plan.execute(
-                named_inputs, outputs, name=self.name, solution_class=solution_class
+                named_inputs,
+                outputs,
+                name=self.name,
+                solution_class=solution_class,
+                layered_solution=layered_solution,
             )
 
             return solution
@@ -372,7 +461,7 @@ class Pipeline(Operation, Plottable):
 
 
 def nest_any_node(ren_args: RenArgs) -> str:
-    """Nest both operation & data under `parent`'s name (if given).
+    """Nest both operation & data under `parent`'s name (if given) but NOT jsonparts.
 
     :return:
         the nested name of the operation or data
@@ -381,7 +470,8 @@ def nest_any_node(ren_args: RenArgs) -> str:
     def prefixed(name):
         return f"{ren_args.parent.name}.{name}" if ren_args.parent else name
 
-    return dep_renamed(ren_args.name, prefixed)
+    if not ren_args.typ.endswith(".jsonpart"):
+        return dep_renamed(ren_args.name, prefixed)
 
 
 def compose(
@@ -444,7 +534,8 @@ def compose(
 
           .. Attention::
               The callable SHOULD wish to preserve any :term:`modifier` on dependencies,
-              and use :func:`.dep_renamed()`.
+              and use :func:`.dep_renamed()` for :attr:`.RenArgs.typ` not ending
+              in ``.jsonpart``.
 
         - If false (default), applies :term:`operation merging`, not *nesting*.
 
@@ -482,7 +573,12 @@ def compose(
     """
     operations = (op1,) + operations
     if not all(isinstance(op, Operation) for op in operations):
-        raise TypeError(f"Non-Operation instances given: {operations}")
+        bad_ops = [op for op in operations if not isinstance(op, Operation)]
+        op_names = [op.name for op in operations if isinstance(op, Operation)]
+        raise TypeError(
+            f"Received x{len(bad_ops)} non-Operation instances: {bad_ops}"
+            f"\n  out of: {op_names}"
+        )
 
     ## Apply default nesting if user asked just a truthy.
     #
