@@ -13,9 +13,9 @@ import pydot
 import pytest
 from jinja2 import Template
 
-from graphtik import base, compose, network, operation, plot
-from graphtik.modifiers import optional
-from graphtik.op import PlotArgs
+from graphtik import base, compose, planning, operation, plot
+from graphtik.modifier import optional
+from graphtik.fnop import PlotArgs
 from graphtik.pipeline import Pipeline
 from graphtik.plot import (
     Plotter,
@@ -67,7 +67,7 @@ def test_plotting_docstring():
     common_formats = ".png .dot .jpg .jpeg .pdf .svg".split()
     for ext in common_formats:
         assert ext in Pipeline.plot.__doc__
-        assert ext in network.Network.plot.__doc__
+        assert ext in planning.Network.plot.__doc__
 
 
 def _striplines(s):
@@ -86,6 +86,8 @@ def test_op_label_template_full():
         color="red",
         fontcolor="blue",
         fillcolor="wheat",
+        op_truncate=plot.Theme.truncate_args,
+        fn_truncate=plot.Theme.truncate_args,
         op_url="http://op_url.com<label>",
         op_tooltip='<op " \t tooltip>',
         op_link_target="_self",
@@ -103,11 +105,11 @@ def test_op_label_template_full():
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="&lt;op &quot; &#9; tooltip&gt;" HREF="http://op_url.com_label_" TARGET="_self"
                 ><FONT COLOR="blue"><B>OP:</B> <I>the op</I></FONT></TD>
                 <TD BORDER="1" SIDES="b"><TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2">
-                        <TR><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#04277d" TITLE="endured" TARGET="_self"
-                            ><FONT FACE="monospace" COLOR="white"><B>!</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#fc89ac" TITLE="rescheduled" TARGET="_self"
-                            ><FONT FACE="monospace" COLOR="white"><B>?</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#b1ce9a" TITLE="parallel" TARGET="_self"
-                            ><FONT FACE="monospace" COLOR="white"><B>|</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#4e3165" TITLE="marshalled" TARGET="_self"
-                            ><FONT FACE="monospace" COLOR="white"><B>&amp;</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#cc5500" TITLE="returns_dict" TARGET="_self"
+                        <TR><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#04277d" TITLE="endured" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-endured" TARGET="_blank"
+                            ><FONT FACE="monospace" COLOR="white"><B>!</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#fc89ac" TITLE="rescheduled" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-partial-outputs" TARGET="_blank"
+                            ><FONT FACE="monospace" COLOR="white"><B>?</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#b1ce9a" TITLE="parallel" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-parallel-execution" TARGET="_blank"
+                            ><FONT FACE="monospace" COLOR="white"><B>|</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#4e3165" TITLE="marshalled" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-marshalling" TARGET="_blank"
+                            ><FONT FACE="monospace" COLOR="white"><B>&amp;</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#cc5500" TITLE="returns_dict" HREF="https://graphtik.readthedocs.io/en/latest/arch.html#term-returns-dictionary" TARGET="_blank"
                             ><FONT FACE="monospace" COLOR="white"><B>}</B></FONT></TD></TR>
                     </TABLE></TD>
             </TR><TR>
@@ -123,7 +125,7 @@ def test_op_label_template_full():
     for k, v in [
         (k, v)
         for k, v in kw.items()
-        if "tooltip" not in k and "url" not in k and "badge" not in k
+        if not any(i in k for i in "truncate tooltip url badge".split())
     ]:
         assert v in got, (k, v)
 
@@ -144,7 +146,13 @@ def test_op_label_template_empty():
 
 
 def test_op_label_template_fn_empty():
-    got = plot._render_template(plot.Theme.op_template, op_name="op", fn_name="fn")
+    got = plot._render_template(
+        plot.Theme.op_template,
+        op_name="op",
+        fn_name="fn",
+        op_truncate=plot.Theme.truncate_args,
+        fn_truncate=plot.Theme.truncate_args,
+    )
     print(got)
     exp = """
         <<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
@@ -246,7 +254,7 @@ def _check_common_end(s1, s2, clip_index):
             assert s1 == s2
 
 
-@active_plotter_plugged(Plotter(include_steps=True))
+@active_plotter_plugged(Plotter(show_steps=True))
 def test_plotters_hierarchy(pipeline: Pipeline, inputs, outputs):
     # Plotting original network, no plan.
     base_dot = str(pipeline.plot(inputs=inputs, outputs=outputs))
@@ -382,9 +390,9 @@ def test_Theme_withset():
     p2 = p.withset()
     assert p is not p2
 
-    p3 = p.withset(include_steps=True)
-    assert p3.include_steps
-    assert not p.include_steps
+    p3 = p.withset(show_steps=True)
+    assert p3.show_steps
+    assert not p.show_steps
 
 
 def test_Plotter_with_styles():
@@ -393,9 +401,9 @@ def test_Plotter_with_styles():
     assert p is not p2
     assert p.default_theme is not p2.default_theme
 
-    p3 = p.with_styles(include_steps=True)
-    assert p3.default_theme.include_steps
-    assert not p.default_theme.include_steps
+    p3 = p.with_styles(show_steps=True)
+    assert p3.default_theme.show_steps
+    assert not p.default_theme.show_steps
 
 
 def test_StylesStack_expansion_ok():
@@ -539,11 +547,11 @@ def test_node_dot_str0(dot_str_pipeline):
         digraph graph_ {
         fontname=italic;
         label=<graph>;
-        <edge> [margin="0.04,0.02", shape=invhouse];
-        <digraph&#58; strict> [margin="0.04,0.02", shape=invhouse];
+        <edge> [label=edge, margin="0.04,0.02", shape=invhouse];
+        <digraph&#58; strict> [label="digraph: strict", margin="0.04,0.02", shape=invhouse];
         <node> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
-                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;node&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;add&#x27;)" TARGET="_top"
+                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FnOp(name=&#x27;node&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;add&#x27;)" TARGET="_top"
                 ><B>OP:</B> <I>node</I></TD>
                 <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
@@ -551,10 +559,10 @@ def test_node_dot_str0(dot_str_pipeline):
                 ><B>FN:</B> &lt;built-in function add&gt;</TD>
             </TR>
         </TABLE>>, shape=plain, tooltip=<node>];
-        <&lt;graph&gt;> [margin="0.04,0.02", shape=house];
+        <&lt;graph&gt;> [label=<graph>, margin="0.04,0.02", shape=house];
         <cu&#58;sto&#58;m> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
-                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET="_top"
+                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FnOp(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET="_top"
                 ><B>OP:</B> <I>cu:sto:m</I></TD>
                 <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
@@ -562,12 +570,12 @@ def test_node_dot_str0(dot_str_pipeline):
                 ><B>FN:</B> test.test_plot.func</TD>
             </TR>
         </TABLE>>, shape=plain, tooltip=<cu:sto:m>];
-        <edge> -> <node>;
-        <edge> -> <cu&#58;sto&#58;m>;
-        <digraph&#58; strict> -> <node>;
-        <digraph&#58; strict> -> <cu&#58;sto&#58;m>;
-        <node> -> <&lt;graph&gt;>;
-        <cu&#58;sto&#58;m> -> <&lt;graph&gt;>;
+        <edge> -> <node>  [arrowtail=inv, dir=back, headport=n, tailport=s];
+        <edge> -> <cu&#58;sto&#58;m>  [arrowtail=inv, dir=back, headport=n, tailport=s];
+        <digraph&#58; strict> -> <node>  [arrowtail=inv, dir=back, headport=n, tailport=s];
+        <digraph&#58; strict> -> <cu&#58;sto&#58;m>  [arrowtail=inv, dir=back, headport=n, tailport=s];
+        <node> -> <&lt;graph&gt;>  [headport=n, tailport=s];
+        <cu&#58;sto&#58;m> -> <&lt;graph&gt;>  [headport=n, tailport=s];
         legend [URL="https://graphtik.readthedocs.io/en/latest/_images/GraphtikLegend.svg", fillcolor=yellow, shape=component, style=filled, target=_blank];
         }
         """
@@ -592,12 +600,12 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
         splines=ortho;
         subgraph "cluster_after pruning" {
         label=<after pruning>;
-        <edge> [fillcolor=wheat, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
-        <digraph&#58; strict> [fillcolor=wheat, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 2"];
-        <&lt;graph&gt;> [fillcolor=SkyBlue, margin="0.04,0.02", shape=house, style=filled, tooltip="(None)"];
+        <edge> [fillcolor=wheat, label=edge, margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
+        <digraph&#58; strict> [fillcolor=wheat, label="digraph: strict", margin="0.04,0.02", shape=invhouse, style=filled, tooltip="(int) 2"];
+        <&lt;graph&gt;> [fillcolor=SkyBlue, label=<graph>, margin="0.04,0.02", shape=house, style=filled, tooltip="(None)"];
         <cu&#58;sto&#58;m> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded" BGCOLOR="wheat">
             <TR>
-                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET="_top"
+                <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FnOp(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET="_top"
                 ><B>OP:</B> <I>cu:sto:m</I></TD>
                 <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
@@ -607,9 +615,9 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
         </TABLE>>, shape=plain, tooltip=<cu:sto:m>];
         }
 
-        <edge> -> <cu&#58;sto&#58;m>;
-        <digraph&#58; strict> -> <cu&#58;sto&#58;m>;
-        <cu&#58;sto&#58;m> -> <&lt;graph&gt;>;
+        <edge> -> <cu&#58;sto&#58;m>  [arrowtail=inv, dir=back, headport=n, tailport=s];
+        <digraph&#58; strict> -> <cu&#58;sto&#58;m>  [arrowtail=inv, dir=back, headport=n, tailport=s];
+        <cu&#58;sto&#58;m> -> <&lt;graph&gt;>  [headport=n, tailport=s];
         legend [URL="https://graphtik.readthedocs.io/en/latest/_images/GraphtikLegend.svg", fillcolor=yellow, shape=component, style=filled, target=_blank];
         }
         """
